<?php

namespace Teknisa\MAN\Controllers;
   
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\Util\Functions;

use Teknisa\MAN\Services\CancelConsVendasService;

class CancelConsVendas extends CrudWithParameterBag {

    protected $dataSourceName = "";
    protected $parameterBag;
    protected $utilService;
    protected $parameterBagColumns = array();

    protected $cancelConsVendasService;

    public function __construct (
            Manager $dataSourceManager, 
            ParameterBag $parameterBag, 
            CancelConsVendasService $cancelConsVendasService, 
            Utilities\Util $utilService) {
        ini_set('max_execution_time', 1800);
        $this->cancelConsVendasService = $cancelConsVendasService;
        $this->utilService = $utilService;
        parent::__construct($dataSourceManager, $parameterBag);
        
    }
    
    public function findVendasCancelEst(Request\Filter $request, Response $response){
        $dataSourceName = $this->dataSourceName = 'vnd07900_cancelconsvendas_s_lstvendas';
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(Functions::arrayKeyExists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			
			if(Functions::arrayKeyExists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}
		
        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        
        foreach ($likeableConditions as $condition) {
            $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
		
        $this->parameterBagColumns = array("DTVENDAFIM", "DTVENDAINI", "TCDCAIXA", "CDCAIXA", "CDFILIAL", "NRORG");

        parent::find($newRequest, $response);
    }
    
    public function findVendasNCancelEst(Request\Filter $request, Response $response){
        self::mountFind($request, $response, 'filterVendasNCancelEst');
    }

    public function findNroLancto(Request\Filter $request, Response $response){
        self::mountFind($request, $response, 'filterNroLancto');
    }
    
    public function findAbaNotaFisc(Request\Filter $request, Response $response){
        self::mountFind($request, $response, 'filterAbaNotaFisc');
    }
    
    public function findAbaVendas(Request\Filter $request, Response $response){
        self::mountFind($request, $response, 'filterAbaVendas');
    }
    
    public function findAbaItensNota(Request\Filter $request, Response $response){
        self::mountFind($request, $response, 'filterAbaItensNota');
    }
    
    public function findAbaItensEst(Request\Filter $request, Response $response){
        self::mountFind($request, $response, 'filterAbaItensEst');
    }
    
    public function findAbaRetirada(Request\Filter $request, Response $response) {  
        
        try {
            $conditions = $this->utilService->getParamsAsArray($request);
        
            $params = array(
                ':NRORG'      => $conditions['NRORG'],
                ':CDFILIAL'   => $conditions['CDFILIAL'],
                ':DTVENDAINI' => $conditions['DTVENDAINI'],
                ':DTVENDAFIM' => $conditions['DTVENDAFIM'],
                ':CDCAIXA'    => $conditions['CDCAIXA'],
                ':TCDCAIXA'   => $conditions['TCDCAIXA'],
                ':LSTVENDAS'  => $conditions['LSTVENDAS'],
                ':TODASVND'   => $conditions['TODASVND']
            );
            
            $result = $this->cancelConsVendasService->find($params);
            
            $response->addDataSet(new DataSet("vnd07900_cancelconsvendas_ret", $result));
            
            return $response;
        } catch (\Exception $e) {
            $response->addDataSet(new DataSet("vnd07900_cancelconsvendas_ret", []));
        }
    }
    
    public function findConsolidacoesConcluidas(Request\Filter $request, Response $response) {  
        $filter = $request->getFilterCriteria();
        $response->addDataSet(new DataSet($this->cancelConsVendasService->findConsolidacoesConcluidas($filter)));
    }
    
    public function findAbaItensRet(Request\Filter $request, Response $response) {
        self::mountFind($request, $response, 'filterAbaItensRet');
        
        $dataSourceName = $this->dataSourceName = 'vnd07900_cancelconsvendas_itens_ret';
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(Functions::arrayKeyExists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(Functions::arrayKeyExists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}
		
        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        
        foreach ($likeableConditions as $condition) {
                $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
		
        $this->parameterBagColumns = array("NRLANCESTQ","CDFILIAL","NRORG");

        parent::find($newRequest, $response);
    }
    
    public function mountFind($request, $response, $dataSource){
        // método copiado do controller de Consolidacao
        $filterCriteria = $request->getFilterCriteria();
        $this->cancelConsVendasService->setParameterBagColumns($this->parameterBagColumns, $filterCriteria->getConditions());
        $this->cancelConsVendasService->setParameterBagValues($this->parameterBag, $filterCriteria->getConditions());
        $this->cancelConsVendasService->setDataSourceName($this->dataSourceName, $dataSource);
        
        $newRequest = $this->cancelConsVendasService->newFilter($request, $this->dataSourceName);
        parent::find($newRequest, $response);
    }
    
    public function getPeriodoContParams(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->dataSourceName = 'getPeriodoContParams';
        
        try {
            $periodContParams = $this->cancelConsVendasService->getPeriodoContParams($row['CDFILIAL'], $row['NRORG']);
            self::responseFrontEnd($response, true, $periodContParams);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }
    
    public function cancelConsolidation(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->dataSourceName = 'cancelConsolidation';
        $cancelConsolidation = array();
        try {
            $cancelConsolidation = $this->cancelConsVendasService->cancelConsolidation($row['toCancel'], $row['filterParams'], $row['dataProc']);
            self::responseFrontEnd($response, true, $cancelConsolidation);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }
    
    public function responseFrontEnd(&$response, $type, $data){
        if($type)
            $response->addDataSet(new DataSet($this->dataSourceName, $data));
        else
            $response->setError(new Response\Error($data->getMessage(), $data->getCode()));
    }
    
}    